local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local c = BittensSpellFlashLibrary

local GetTime = GetTime
local MAX_POWER_PER_EMBER = MAX_POWER_PER_EMBER
local SPELL_POWER_BURNING_EMBERS = SPELL_POWER_BURNING_EMBERS
local SPELL_POWER_DEMONIC_FURY = SPELL_POWER_DEMONIC_FURY
local SPELL_POWER_SOUL_SHARDS = SPELL_POWER_SOUL_SHARDS
local UnitPower = UnitPower
local math = math
local select = select

a.Rotations = {}
c.RegisterForEvents(a)
a.SetSpamFunction(function()
	c.Flash(a)
end)

local function flashSummon(...)
	if c.HasTalent("Grimoire of Sacrifice") then
		if not c.SelfBuffNeeded("Grimoire of Sacrifice") then
			return
		end
		
		if x.PetAlive then
			s.Flash(c.GetID("Grimoire of Sacrifice"), "yellow")
			return
		end
	elseif x.PetAlive then
		return
	end
	
--	local current = a.GetCurrentPet()
--	for i = 1, select("#", ...) do
--		if select(i, ...) == current then
--			return
--		end
--	end
	
	s.Flash(c.GetID("Summon Demon"), "yellow")
	for i = 1, select("#", ...) do
		s.Flash(c.GetID("Summon " .. select(i, ...)), "yellow")
	end
end

-------------------------------------------------------------------- Affliction
a.Rotations.Afliction = {
	Spec = 1,
	OffSwitch = "affliction_off",
	
	FlashInCombat = function()
		a.Shards = s.Power("player", SPELL_POWER_SOUL_SHARDS)
		local info = c.GetQueuedInfo()
		if c.InfoMatches(info, "Soulburn") or c.IsCasting("Haunt") then
			a.Shards = a.Shards - 1
		end
		
		c.FlashAll(
			"Curse of the Elements",
			"Dark Soul: Misery",
			"Soulburn under Dark Soul: Misery",
			"Grimoire: Felhunter",
			"Spell Lock",
			"Optical Blast",
			"Command Spell Lock")
		if s.HealthPercent() <= 20 then
			c.PriorityFlash(
				"Haunt",
				"Soulburn during Execute",
				"Soul Swap during Execute",
				"Life Tap for Affliction",
				"Drain Soul")
		else
c.Debug("Flash", a.Shards, 
			c.PriorityFlash(
				"Haunt",
				"Agony",
				"Corruption",
				"Unstable Affliction",
				"Agony with Pandemic",
				"Corruption with Pandemic",
				"Unstable Affliction with Pandemic",
				"Life Tap for Affliction",
				"Malefic Grasp")
)
		end
	end,
	
	FlashAlways = function()
		c.FlashAll(
			"Soul Swap under Soulburn", 
			"Dark Intent", 
			"Soulstone", 
			"Dark Regeneration")
		flashSummon("Felhunter", "Observer")
	end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(info, "Soulburn") then
			a.SoulburnedDuringMisery = s.Buff(
				c.GetID("Dark Soul: Misery"), "player")
			c.Debug(
				"Event", "Soulburned during Misery:", a.SoulburnedDuringMisery)
		elseif c.InfoMatches(info, "Dark Soul: Misery") then
			a.SoulburnedDuringMisery = false
			c.Debug("Event", "Misery Cast")
		end
	end,
	
	AuraApplied = function(spellID)
		
		-- Do this here instead of c.ManageDotRefresh so that it updates when
		-- cast via Soul Swap
		if spellID == c.GetID("Corruption") then
			local tick = c.GetHastedTime(2)
			c.GetSpell("Corruption").EarlyRefresh = tick
			c.Debug("Event", "Corruption ticks every", tick)
		elseif spellID == c.GetID("Unstable Affliction") then
			local tick = c.GetHastedTime(2)
			c.GetSpell("Unstable Affliction").EarlyRefresh = tick
			c.Debug("Event", "Unstable Affliction ticks every", tick)
		end
	end,
}

-------------------------------------------------------------------- Demonology
a.Rotations.Demonology = {
	Spec = 2,
	OffSwitch = "demonology_off",
	
	FlashInCombat = function()
		a.Fury = s.Power("player", SPELL_POWER_DEMONIC_FURY)
		a.Morphed = s.Form(c.GetID("Metamorphosis"))
		c.FlashAll(
			"Curse of the Elements",
			"Dark Soul: Knowledge",
			"Felstorm",
			"Wrathstorm",
			"Grimoire: Felguard",
			"Axe Toss",
			"Super Axe Toss")
c.Debug("Flash", a.Fury, 
		c.PriorityFlash(
			"Corruption for Demonology",
			"Doom",
			"Metamorphosis",
			"Metamorphosis Cancel",
			"Hand of Gul'dan",
			"Touch of Chaos for Corruption",
			"Soul Fire",
			"Touch of Chaos",
			"Life Tap under 50",
			"Shadow Bolt")
)
	end,
	
	FlashAlways = function()
		c.FlashAll("Dark Intent", "Soulstone", "Dark Regeneration")
		flashSummon("Felguard", "Wrathguard")
	end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(info, "Doom") then
			a.DoomIsBuffed = c.HasBuff("Dark Soul: Knowledge")
			c.Debug("Event", "Doom is buffed:", a.DoomIsBuffed)
		end
	end,
	
	LeftCombat = function()
		a.DoomIsBuffed = false
		c.Debug("Event", "Left combat")
	end,
}

------------------------------------------------------------------- Destruction
local lastEmbers = nil
local pendingEmberBump = 0
local pendingEmberDrop = false

a.Rotations.Destruction = {
	Spec = 3,
	OffSwitch = "destro_off",
	
	FlashInCombat = function()
		a.Embers = UnitPower("player", SPELL_POWER_BURNING_EMBERS, true) 
			/ MAX_POWER_PER_EMBER
		if lastEmbers ~= nil then
			if a.Embers > lastEmbers then
				pendingEmberBump = math.max(
					0, pendingEmberBump - (a.Embers - lastEmbers))
				c.Debug("Flash", "Ember bump occurred")
			elseif a.Embers < lastEmbers then
				pendingEmberDrop = false
				c.Debug("Flash", "Ember drop occurred")
			end
		end
		lastEmbers = a.Embers
		a.Embers = a.Embers + pendingEmberBump
		if pendingEmberDrop then
			a.Embers = a.Embers - 1
		end
		
		if c.IsCasting("Incinerate") or c.IsCasting("Conflagrate") then
			a.Embers = a.Embers + .1
		elseif c.IsCasting("Chaos Bolt") 
			or c.IsCasting("Shadowburn") 
			or c.IsCasting("Fire and Brimstone") then
			
			a.Embers = a.Embers - 1
		end
		
		a.Backdraft = c.GetBuffStack("Backdraft")
		if a.Backdraft > 0 and c.IsCasting("Incinerate") then
			a.Backdraft = a.Backdraft - 1
		elseif c.IsCasting("Conflagrate") then
			a.Backdraft = 3
		end
		
		
		c.FlashAll(
			"Curse of the Elements",
			"Dark Soul: Instability",
			"Grimoire: Felhunter",
			"Ember Tap",
			"Spell Lock",
			"Optical Blast",
			"Command Spell Lock")
		if c.AoE then
			local flashing = c.PriorityFlash(
				"Immolate",
				"Rain of Fire",
				"Conflagrate",
				"Incinerate")
			if flashing == "Immolate" 
				or (a.Embers >= 2 and flashing == "Incinerate") then
				
				c.PriorityFlash("Fire and Brimstone")
			end
		else
c.Debug("Flash", a.Embers, a.Backdraft, pendingEmberBump, pendingEmberDrop,
			c.PriorityFlash(
				"Shadowburn",
				"Immolate",
				"Chaos Bolt",
				"Conflagrate",
				"Incinerate")
)
		end
	end,
	
	FlashAlways = function()
		c.FlashAll("Dark Intent", "Soulstone", "Dark Regeneration")
		flashSummon("Felhunter", "Observer")
	end,
	
--CastQueued = function(info)
--c.Debug("Event", "Queued", info.Name)
--end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(
			info, 
			"Incinerate", 
			"Conflagrate", 
			"Incinerate AoE", 
			"Conflagrate AoE") then
			
			pendingEmberBump = pendingEmberBump + .1
			c.Debug("Event", "Ember bump pending:", info.Name)
		elseif c.InfoMatches(
			info, "Chaos Bolt", "Shadowburn", "Fire and Brimstone") then
			
			pendingEmberDrop = true
			c.Debug("Event", "Ember drop pending:", info.Name)
		end
	end,
	
	SpellDamage = function(spellID, target, amount, critical)
--c.Debug("Event", "Damage:", spellID, s.SpellName(spellID), target, amount, critical)
		if critical 
			and (spellID == c.GetID("Incinerate")
				or spellID == c.GetID("Conflagrate")
				or spellID == c.GetID("Immolate")
				or spellID == c.GetID("Incinerate AoE")
				or spellID == c.GetID("Conflagrate AoE")
				or spellID == c.GetID("Immolate AoE")) then
			
			pendingEmberBump = pendingEmberBump + .1
			c.Debug("Event", "Ember bump pending from crit")
		end
	end,
	
	LeftCombat = function()
		lastEmbers = nil
		pendingEmberBump = 0
		pendingEmberDrop = false
		c.Debug("Event", "Left combat")
	end,
}
